import 'dart:convert';

import 'package:crypto/crypto.dart' as crypto;
import 'package:firebase_auth/firebase_auth.dart' as fb;
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/auth_icon_font.dart';

import 'login_provider.dart';

class AppleLoginProvider extends LoginProvider {
  final Future Function(BuildContext context)? onWeb;
  const AppleLoginProvider({this.onWeb});

  static final oAuthProvider = fb.OAuthProvider('apple.com');

  final String name = "apple";
  static final _log = Logger("appleLoginHandler");

  @override
  Widget logo(double size) {
    return Icon(
      AuthIcons.appleLogo,
      color: Colors.white,
      size: size,
    );
  }

  @override
  Future signOut() async {
    /// This plugin doesn't cache anything, so nothing to do.
  }

  @override
  Future<AuthResponse> doAttemptLogin(BuildContext context) async {
    try {
      try {
        if (kIsWeb) {
          await onWeb?.call(context);
          return AuthResponse.cancelled();
        }
        final String nonce = uuid();

        final shaOfNonce = crypto.sha256
            .convert(utf8.encode(nonce))
            .toString()
            .replaceAll("-", '');

        final credential = await SignInWithApple.getAppleIDCredential(
          scopes: [
            AppleIDAuthorizationScopes.email,
            AppleIDAuthorizationScopes.fullName,
          ],
          nonce: shaOfNonce,
        );

        final oauthCred = oAuthProvider.credential(
          idToken: credential.identityToken,
          accessToken: credential.authorizationCode,
          rawNonce: nonce,
        );

        /// Apple only allows one access with the token, so don't do anything with
        /// the token before sending it to Firebase
        return AuthResponse.success(oauthCred);
      } catch (e, stack) {
        _log.severe("Error from server: $e", e, stack);
        return AuthResponse.error(e);
      }
    } catch (e, stack) {
      _log.severe("Error from server: $e", e, stack);
      return AuthResponse.error("There was an error logging you in");
    }
  }
}
