import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/routing2/frouter_delegate.dart';

AppRouteMatch rootAppRouteMatch(Widget splashPage()) => AppRouteMatch(
      AppPageRoute('/', (context, _) => splashPage(), (_) => null),
      const {},
    );

class FRouterInformationParser
    with LoggingMixin
    implements RouteInformationParser<FRouteMatch> {
  /// Used to find other routers that might handle this route
  final FRouterDelegate delegate;
  final FRouter router;

  const FRouterInformationParser(this.router, this.delegate);

  String get loggerName => "routeParser";

  @override
  Future<FRouteMatch> parseRouteInformation(
      RouteInformation routeInformation) async {
    return delegate.matchRoute(routeInformation);
  }

  @override
  RouteInformation restoreRouteInformation(FRouteMatch configuration) {
    var routeUri =
        configuration.match.route?.routeUri(configuration.match.parameters);

    log.info("restore $routeUri");
    if (routeUri == null) {
      return RouteInformation(location: '/');
    } else {
      return RouteInformation(
        state: configuration.match.parameters,
        location: routeUri,
      );
    }
  }
}

class FRouteMatch extends Equatable {
  final AppRouteMatch match;
  final FRouterDelegate matchedBy;
  final bool isManaged;

  FRouteMatch.page(this.match, this.matchedBy) : isManaged = true;

  /// A route not tracked by the FRouter (pushed and popped separately)
  FRouteMatch.external(this.matchedBy)
      : match = AppRouteMatch(ExternalAppRoute(), {}),
        isManaged = false;

  @override
  List<Object?> get props => [match, matchedBy.path, isManaged];
}

class ExternalAppRoute implements AppRoute {
  final name = "/${uuid}";

  @override
  String get route => name!;

  @override
  String? routeTitle([RouteParams? params]) => "External route";

  @override
  String? routeUri(params) => name;

  @override
  ToRouteUri? get toRouteUri => null;

  @override
  ParameterConverter? get paramConverter => null;
}
